# -*- coding: utf-8 -*-
from .path import *
from .core import *

from .core import color as core_color  # avoid conflict with arg name


# ==============================================================================
# ==============================================================================
# ==============================================================================
# ==============================================================================
# ==============================================================================
# ==============================================================================
# ==============================================================================
# ==============================================================================
def write(text="Write something here", style="body", x=0, y=0, size=1.0, rotate=0, color="black", background=None, outline=False, outline_size=0.1, outline_color="black", allow=None, wait=None, long_text=False, fast=False):
    """
    Display some text on screen.

    Parameters
    -------------
    text : str
        The text to display.
    style : str
        'body', 'psychometry', 'psychometry_bold', 'light', 'bold', 'title', 'subtitle' or 'end'. Can overwrite other parameters such as position, size or allow. You can also insert the name of a system font, or the path to a specific font.
    x : float
        Horizontal position of the center (from -10 (left) to 10 (right)).
    y : float
        Vertical position of the center (from -10 (bottom) to 10 (top)).
    size : float
        Text size.
    rotate : int
        Rotation angle (0 to 360).
    color : str or tuple
        Text color. See `neuropsydia.color()`.
    background : str
        Background color. See `neuropsydia.color()`.
    outline : bool
        Text outline (unperfect for now, as the outline is larger for horizontal than for vertical lines).
    outline_size : float
        Outline size.
    outline_color : str or tuple
        Outline color. See `neuropsydia.color()`.
    allow : str or list
        Wait until a specific key is pressed (e.g., 'ENTER', ['LEFT', 'RIGHT'] or 'any').
    wait : float
        Wait time (in milliseconds).
    long_text : bool
        Set to True for longer texts on multiple lines. Then, the x and y parameters are not working, but you can jump lines using 'backslash + n' in your text. Some parameters are disabled. Unperfect for now.
    fast : bool
        Disables some parameters for improved speed.


    Example
    ----------
    >>> import neuropsydia as n
    >>> n.start()
    >>> n.write('here is my title', style='title')
    >>> n.write('here is my text', font_color='red')
    >>> n.write('press ENTER to quit', style='end')
    >>> n.close()

    Notes
    ----------
    *Authors*

    - Dominique Makowski (https://github.com/DominiqueMakowski)
    - Léo Dutriaux (https://github.com/LeoDutriaux)

    *Dependencies*

    - pygame
    """
    if fast is True:
        size = int(size*screen_width/35.0)
        x, y = Coordinates.to_pygame(x=x, y=y)
        if style != 'body':
            font = Font.get(style, size)
        else:
            font = Font.get('RobotoRegular.ttf', size)
        surface = font.render(text, True, core_color(color))
        rectangle = surface.get_rect()
        rectangle.center = (x, y)
        screen.blit(surface, rectangle)

    else:
        size = int(size*screen_width/35.0)
        outline_size = int(outline_size*screen_width/35.0)
        text = str(text)

        if style == 'body':
            font_name = Path.font() + 'RobotoRegular.ttf'
        elif style == 'psychometry':
            font_name = Path.font() + 'LiberationMono-Regular.ttf'
        elif style == 'psychometry_bold':
            font_name = Path.font() + 'LiberationMono-Bold.ttf'
        elif style == 'light':
            font_name = Path.font() + 'RobotoLight.ttf'
        elif style == 'bold':
            font_name = Path.font() + 'RobotoBold.ttf'
        elif style == 'title':
            if size == int(1.0*screen_width/35.0):
                size = int(2.0*screen_width/35.0)
            font_name = Path.font() + 'RobotoBlack.ttf'
            if y == 0:
                y = 8.5
            if background==None:
                background="white"
        elif style == 'subtitle':
            if size == int(1.0*screen_width/35.0):
                size = int(1.5*screen_width/35.0)
            font_name = Path.font() + 'RobotoBold.ttf'
            if y == 0:
                y=7
        elif style == 'end':
            font_name = Path.font() + 'RobotoBold.ttf'
            if y == 0:
                y = -9
            if allow==None:
                allow="ENTER"
        else:
            font_name = style

        font = Font.get(font_name,size)
        if outline==True:
            font_outline = Font.get(font_name, size+outline_size)

        #If NOT long text
        if long_text is False:
            x, y = Coordinates.to_pygame(x=x,y=y)
            if outline is True:
                surface = font_outline.render(text, True, core_color(outline_color))
                if rotate != None or rotate != 0:
                    surface = pygame.transform.rotate(surface,rotate)
                rectangle = surface.get_rect()
                rectangle.center = (x,y)
                newpage(background, auto_refresh=False)
                screen.blit(surface, rectangle)
            surface = font.render(text, True, core_color(color))
            if rotate != None or rotate != 0:
                surface = pygame.transform.rotate(surface,rotate)
            rectangle = surface.get_rect()
            rectangle.center = (x,y)
            newpage(background, auto_refresh=False)
            screen.blit(surface, rectangle)



        #If long text
        if long_text is True:
            paragraph = ''
            line = 0
            for i in range(len(text)):
                if i == (len(text) - 1):
                    paragraph = paragraph + text[i]
                    surface = font.render(paragraph, 1, (0, 0, 0))
                    rectangle = surface.get_rect()
                    rectangle.center = (screen_width/2, (surface.get_height()/2)+(line*surface.get_height()))
                    screen.blit(surface, rectangle)

                elif text[i] == '\n':
                    surface = font.render(paragraph, 1, (0, 0, 0))
                    rectangle = surface.get_rect()
                    rectangle.center = (screen_width/2, (surface.get_height()/2)+(line*surface.get_height()))
                    screen.blit(surface, rectangle)
                    paragraph = ""
                    line += 1

                else:
                    paragraph = paragraph + text[i]
                    if text[i] == " ":
                        buffer = paragraph
                        for x in range (len(text)-i-1):
                            if text[i+x+1] == '\n' or text[i+x+1]== " ":
                                break
                            else:
                                buffer = buffer+text[i+x+1]
                        surface_buffer = font.render(buffer, 1, (0, 0, 0))
                        rectangle = surface_buffer.get_rect()
                        if surface_buffer.get_width()+screen_width/4>screen_width:
                            surface = font.render(paragraph, 1, (0, 0, 0))
                            rectangle = surface.get_rect()
                            rectangle.center = (screen_width/2, (surface.get_height()/2)+(line*surface.get_height()))
                            screen.blit(surface, rectangle)
                            paragraph = ""
                            line += 1





        #END : In case we must wait until something
        if allow != None or wait != None:
            refresh()
            if allow == None and wait != None:
                time.wait(wait)
                return(wait)
            if allow != None:
                return(response(allow=allow,time_max=wait))



# -*- coding: utf-8 -*-
from .screen import screen, screen_width, screen_height, monitor_diagonal

import os
import pygame
import time as builtin_time
import numpy as np
import datetime


# ==============================================================================
# ==============================================================================
# ==============================================================================
# ==============================================================================
# ==============================================================================
# ==============================================================================
# ==============================================================================
# ==============================================================================
class Time():
    """
    A class object to wait some time, get time, control time and such.
    Its methods (functions) are:
        - reset()
        - control()
        - wait()
        - get()
    See those for further informations.

    Note that by default, there is already a Time class object called "time" (lowercase) that is initialized at neuropsydia's loading. For the sake of clarity, use this one (e.g., n.time.wait() ), especially for wait() and control() functions.

    Parameters
    ----------
    None

    Returns
    ----------
    None

    Example
    ----------
    >>> import neuropsydia as n
    >>> n.start()
    >>> myclock = n.Time()
    >>> time_passed_since_myclock_creation = myclock.get()
    >>> myclock.reset()
    >>> time_passed_since_reset = myclock.get()
    >>> n.close()

    Authors
    ----------
    Dominique Makowski

    Dependencies
    ----------
    - pygame
    - time
    """
    def __init__(self):
        self.clock = builtin_time.perf_counter()
        self.pygame_clock = pygame.time.Clock()

    def reset(self):
        """
        Reset the clock of the Time object.

        Parameters
        ----------
        None

        Returns
        ----------
        None

        Example
        ----------
        >>> import neuropsydia as n
        >>> n.start()
        >>> time_passed_since_neuropsydia_loading = n.time.get()
        >>> n.time.reset()
        >>> time_passed_since_reset = n.time.get()
        >>> n.close()

        Authors
        ----------
        Dominique Makowski

        Dependencies
        ----------
        - pygame
        - time
        """
        self.clock = builtin_time.perf_counter()
        self.pygame_clock = pygame.time.Clock()

    def control(self, frequency=60):
        """
        Control time. Must be placed in a while loop and, each time the program runs through it, checks if the time passed is less than a certain amount (the frequency, by default 60, so 1/60 seconds). If true, the program stops and wait what needed before continuing, so that each loop takes at least 1/frequency seconds to be complete.

        Parameters
        ----------
        frequency = int, optional
            The minimum frequency you want the loop to run at

        Returns
        ----------
        None

        Example
        ----------
        >>> import neuropsydia as n
        >>> n.start()
        >>> while n.time.get() < 5:
        >>>     n.time.control()
        >>>     print(n.time.get())
        >>> n.close()

        Authors
        ----------
        Dominique Makowski

        Dependencies
        ----------
        - pygame
        - time
        """
        self.pygame_clock.tick_busy_loop(frequency)

    def get(self, reset=True):
        """
        Get time since last initialisation / reset.

        Parameters
        ----------
        reset = bool, optional
            Should the clock be reset after returning time?

        Returns
        ----------
        float
            Time passed in milliseconds.

        Example
        ----------
        >>> import neuropsydia as n
        >>> n.start()
        >>> time_passed_since_neuropsydia_loading = n.time.get()
        >>> n.time.reset()
        >>> time_passed_since_reset = n.time.get()
        >>> n.close()

        Authors
        ----------
        Dominique Makowski

        Dependencies
        ----------
        - pygame
        - time
        """
        t = (builtin_time.perf_counter()-self.clock)*1000

        if reset is True:
            self.reset()
        return(t)

    def wait(self, time_to_wait, unit="ms", frequency=60, round_by_frame=True, skip=None):
        """
        Wait some time.

        Parameters
        ----------
        time_to_wait = int
            Time to wait
        unit = str
            "min" for minutes, "s" for seconds, "ms" for milliseconds, or "frame" for a certain amount of frames (depending on the frequency parameter)
        frequency = int
            should be a multiple of your monitor's refresh rate
        round by frame = bool
            should the waiting time be rounded to match an exact number of frame / refresh cycles? (e.g., on a 60Hz monitor, 95ms will be rounded to 100, because the monitor is refreshed every 16.6667ms)
        skip = str
            Shoud there be a key to skip the waiting. Default to None.

        Returns
        ----------
        float
            Actual time waited in milliseconds

        Example
        ----------
        >>> import neuropsydia as n
        >>> n.start()

        >>> n.write("let's wait 500ms", round_by_frame = False)
        >>> n.refresh()
        >>> wait_time = n.time.wait(520)
        >>> n.newpage("white")
        >>> n.write("I waited for " + str(wait_time) + "ms")
        >>> n.refresh()
        >>> wait_time = n.time.wait(520, round_by_frame = True)
        >>> n.newpage("white")
        >>> n.write("I waited for " + str(wait_time) + "ms")
        >>> n.refresh()
        >>> n.time.wait(3, unit = "s")

        >>> n.close()

        Authors
        ----------
        Dominique Makowski

        Dependencies
        ----------
        - pygame
        - time
        """
        t0 = builtin_time.perf_counter()
        if unit == "min":
            time_to_wait = time_to_wait * 60
            unit = "s"
        if unit == "s":
            time_to_wait = time_to_wait * 1000
            unit = "ms"
        if unit == "ms":
            if round_by_frame is True:
                time_to_wait = round(time_to_wait / (1/frequency*1000))
                time_to_wait = round(time_to_wait * (1/frequency*1000))
        if unit == "frame":
            time_to_wait = time_to_wait * (1/frequency*1000)

        if skip is None:
            pygame.time.delay(time_to_wait)  # In milliseconds
        else:
            response(allow=skip, time_max=time_to_wait)
        return((builtin_time.perf_counter()-t0)*1000)

    def now(self):
        """
        Returns current (absolute) date and time.

        Parameters
        ----------
        None

        Returns
        ----------
        datetime
            Current date and time.

        Example
        ----------
        >>> import neuropsydia as n
        >>> n.start()

        >>> n.time.now()

        >>> n.close()

        Authors
        ----------
        Dominique Makowski

        Dependencies
        ----------
        - datetime
        """
        return(datetime.datetime.now())

# Initialize a Time() object.
time = Time()

# ==============================================================================
# ==============================================================================
# ==============================================================================
# ==============================================================================
# ==============================================================================
# ==============================================================================
# ==============================================================================
# ==============================================================================


def newpage(color_name="white", opacity=100, fade=False, fade_speed=60, fade_type="out", auto_refresh=True):
    """
    Fill the background with a color.

    Parameters
    ------------
        color_name : str, tuple, optional
            name of the color (see color() function), or an RGB tuple (e.g., (122,84,01)).
        opacity : int, optional
            opacity of the color (in percents).
        fade : bool, optional
            do you want a fade effect?
        fade_speed : int, optional
            frequency (speed) of the fading.
        fade_type : str, optional
            "out" or "in", fade out or fade in.


    Example
    ----------
    >>> import neuropsydia as n
    >>> n.start()
    >>> n.newpage("blue")
    >>> n.refresh()
    >>> n.time.wait(500)
    >>> n.close()

    Notes
    ----------
    *Authors*

    - Dominique Makowski (https://github.com/DominiqueMakowski)

    *Dependencies*

    - pygame
    - time
    """
    if color_name is not None:
        if fade is False:
            if opacity == 100:
                try:
                    screen.fill(color(color_name))
                except:
                    print("NEUROPSYDIA ERROR: newpage(): wrong argument")
            else:
                opacity = int(opacity * 255 / 100)
                color_name = color(color_name) + (opacity,)
                mask = pygame.Surface((screen_width, screen_height), pygame.SRCALPHA)  # per-pixel alpha
                mask.fill(color_name)  # notice the alpha value in the color
                screen.blit(mask, (0, 0))
            if auto_refresh is True:
                refresh()
        if fade is True:
            original_color_name = color_name
            clock = pygame.time.Clock()
            for i in range(0, 40, 1):
                clock.tick_busy_loop(fade_speed)
                if fade_type == "out":
                    color_name = color(original_color_name) + (i,)
                else:
                    color_name = color(original_color_name) + (255-i, )
                mask = pygame.Surface((screen_width, screen_height), pygame.SRCALPHA)  # per-pixel alpha
                mask.fill(color_name)  # notice the alpha value in the color
                screen.blit(mask, (0, 0))
                refresh()
            screen.fill(color(original_color_name))


# ==============================================================================
# ==============================================================================
# ==============================================================================
# ==============================================================================
# ==============================================================================
# ==============================================================================
# ==============================================================================
# ==============================================================================


def refresh():
    """
    Refresh / flip the screen: actually display things on screen.


    Example
    ----------
    >>> import neuropsydia as n
    >>> n.start()
    >>> n.newpage("blue")
    >>> n.refresh()
    >>> n.time.wait(500)
    >>> n.close()

    Notes
    ----------
    *Authors*

    - Dominique Makowski (https://github.com/DominiqueMakowski)

    *Dependencies*

    - pygame
    """
    pygame.display.flip()

# ==============================================================================
# ==============================================================================
# ==============================================================================
# ==============================================================================
# ==============================================================================
# ==============================================================================
# ==============================================================================
# ==============================================================================
keys = {
"normal":
    {109: ',', 44: ';', 46: ':',47: '!', 97: 'q',
     98: 'b', 99: 'c', 100: 'd', 101: 'e',
     102: 'f', 103: 'g', 104: 'h', 105: 'i', 106: 'j',
     107: 'k', 108: 'l', 59: 'm', 110: 'n', 111: 'o',
     112: 'p', 113: 'a', 114: 'r', 115: 's', 116: 't',
     117: 'u', 118: 'v', 119: 'z', 120: 'x', 121: 'y',
     122: 'w', 57: 'ç', 56: '_', 55: 'è', 54: '-', 53: '(',
     52: "_", 51: '_', 50:'é', 49: '&', 48: 'à', 32: 'SPACE',
     13: "ENTER",276: "LEFT",274: "DOWN",275: "RIGHT",273: "UP",
     266: ".",
     pygame.K_KP0: '0', pygame.K_KP1: '1', pygame.K_KP2: '2', pygame.K_KP3: '3',pygame.K_KP4: '4', pygame.K_KP5: '5',pygame.K_KP6: '6',pygame.K_KP7: '7',pygame.K_KP8: '8',pygame.K_KP9: '9'},

"shift":
    {109:'?', 44: '.', 46: '/',47: '§', 97: 'Q',
     98: 'B', 99: 'C', 100: 'D', 101 : 'E',
     102: 'F', 103: 'G', 104: 'H', 105: 'I', 106: 'J',
     107: 'K', 108: 'L', 59: 'M', 110: 'N', 111: 'O',
     112: 'P', 113: 'A', 114: 'R', 115: 'S', 116: 'T',
     117: 'U', 118: 'V', 119: 'Z', 120: 'X', 121: 'Y',
     122: 'W', 57: '9', 56: '8', 55: '7', 54: '6', 53: '5',
     52: '4', 51: '3', 50: '2', 49: '1', 48: '0', 32: 'SPACE',
     13: "ENTER",276: "LEFT",274: "DOWN",275: "RIGHT",273: "UP",
     266: ".",
     pygame.K_KP0: '0',pygame.K_KP1: '1',pygame.K_KP2: '2',pygame.K_KP3:'3',pygame.K_KP4: '4',
     pygame.K_KP5: '5',pygame.K_KP6: '6',pygame.K_KP7: '7',pygame.K_KP8: '8',pygame.K_KP9:'9'},

"altgr":
    {97: 'q', 98: 'b', 99: 'c', 100: 'd', 101 : '€',
     102: 'f', 103: 'g', 104: 'h', 105: 'i', 106: 'j',
     107: 'k', 108: 'l', 59: 'm', 110: 'n', 111: 'o',
     112: 'p', 113: 'a', 114: 'r', 115: 's', 116: 't',
     117: 'u', 118: 'v', 119: 'z', 120: 'x', 121: 'y',
     122: 'w', 57: '^', 56: '_', 55: '`', 54: '|', 53: '[',
     52: '{', 51: '#', 50: '~', 49: '1', 48: '@', 32: 'SPACE',
     13: "ENTER",276: "LEFT",274: "DOWN",275: "RIGHT",273: "UP",
     266: ".",
     pygame.K_KP0: '0',pygame.K_KP1: '1',pygame.K_KP2: '2',pygame.K_KP3: '3',pygame.K_KP4: '4',
     pygame.K_KP5: '5',pygame.K_KP6: '6',pygame.K_KP7: '7',pygame.K_KP8: '8',pygame.K_KP9: '9'}
}


def wait_for_input(time_max=None):
    """
    Low level input checker.

    Parameters
    ----------
    time_max = int
        time max in ms

    Returns
    ----------
    key
        A key.
    Example
    ----------
    NA

    Authors
    ----------
    Dominique Makowski

    Dependencies
    ----------
    - pygame
    - time
    """
    if pygame.event.get_blocked(pygame.KEYDOWN) is True:
        blocked = True
        pygame.event.set_allowed(pygame.KEYDOWN)
    else:
        blocked = False
        pygame.event.set_allowed(pygame.KEYDOWN)

    time_out = False
    loop = True
    if time_max is None:
        while loop:
            for event in pygame.event.get():
                #print(event)
                if event.type == pygame.KEYDOWN:
                    if pygame.key.get_mods() & pygame.KMOD_SHIFT:
                        modifier = "shift"
                    elif pygame.key.get_mods() & pygame.KMOD_RALT:
                        modifier = "altgr"
                    else:
                        modifier = "normal"
                    if event.key != pygame.K_RSHIFT and event.key != pygame.K_LSHIFT and event.key != pygame.K_RALT:
                        loop = False
    else:
        local_time = Time()
        while loop and local_time.get(reset=False) < time_max:
            for event in pygame.event.get():
                if event.type == pygame.KEYDOWN:
                    if pygame.key.get_mods() & pygame.KMOD_SHIFT:
                        modifier = "shift"
                    elif pygame.key.get_mods() & pygame.KMOD_RALT:
                        modifier = "altgr"
                    else:
                        modifier = "normal"
                    if event.key != pygame.K_RSHIFT and event.key != pygame.K_LSHIFT and event.key != pygame.K_RALT:
                        loop = False
        if local_time.get(reset=False) > time_max:
            time_out = True


    if blocked is True:
        pygame.event.set_blocked(pygame.KEYDOWN)

    if time_out == True:
        return("Time_Max_Exceeded")
    
    else:
        try:
            #print("USED")
            print(event)
            return(keys[modifier][event.key])
        except (KeyError, AttributeError):
            #print("ERROR")
            #print(event)
            return(event.key)


def response(allow=None, enable_escape=True, time_max=None, get_RT=True):
    """
    Get a (keyboard, for now) response.

    Parameters
    ----------
    allow : str or list
        Keys to allow.
    enable_escape : bool
        Enable escape key to exit.
    time_max : int
        Maximum time to wait for a response (ms).
    get_RT : bool
        Return response time.

    Returns
    ----------
    str or tuple
        returns a tuple when get_RT is set to True

    Notes
    ----------
    *Authors*

    - Dominique Makowski (https://github.com/DominiqueMakowski)

    *Dependencies*

    - pygame
    """
    local_time = Time()
    if allow is not None:
        if not isinstance(allow, list):
            allow = [allow]

    while True:
        pressed_key = wait_for_input(time_max=time_max)
        if pressed_key == "Time_Max_Exceeded":
            return("Time_Max_Exceeded", local_time.get())
        if pressed_key == pygame.K_ESCAPE:
            if enable_escape is True:
                if get_RT is True:
                    return("ESCAPE", local_time.get())
                else:
                    return("ESCAPE")
        elif allow is not None:
            if pressed_key in allow:
                if get_RT is True:
                    return(pressed_key, local_time.get())
                else:
                    return(pressed_key)
        else:
            if get_RT is True:
                return(pressed_key, local_time.get())
            else:
                return(pressed_key)



# ==============================================================================
# ==============================================================================
# ==============================================================================
# ==============================================================================
# ==============================================================================
# ==============================================================================
# ==============================================================================
# ==============================================================================
class Coordinates:
    """
    A class object to go from pygame corrdinates system to neuropsydia's and vice versa.

    Its methods (functions) are:
        - to_pygame()
        - from_pygame()

    Parameters
    ----------
    None

    Returns
    ----------
    None

    Example
    ----------
    None

    Authors
    ----------
    Dominique Makowski

    Dependencies
    ----------
    - pygame
    """
    def to_pygame(x=None, y=None, distance_x=None, distance_y=None):
        """
        Convert coordinates from neuropsydia (-10:10) to pygame's system (in pixels).

        Parameters
        ----------
        x = float
            [-10:10]
        y = float
            [-10:10]
        distance_x = convert a horizontal distance
            [-10:10]
        distance_y = convert a horizontal distance
            [-10:10]
        Returns
        ----------
        NA

        Example
        ----------
        NA

        Authors
        ----------
        Dominique Makowski

        Dependencies
        ----------
        - pygame
        """
        if x != None and y == None:
            x = (x+10.0)/(10.0+10.0)*(screen_width-0.0)+0.0
            return(int(x))
        if x == None and y != None:
            y = (-y+10.0)/(10.0+10.0)*(screen_height-0.0)+0.0
            return(int(y))
        if x != None and y != None:
            x = (x+10.0)/(10.0+10.0)*(screen_width-0.0)+0.0
            y = (-y+10.0)/(10.0+10.0)*(screen_height-0.0)+0.0
            return(int(x),int(y))
        if distance_x != None and distance_y is None:
            distance_x = (distance_x)/(10.0+10.0)*(screen_width-0.0)+0.0
            return(int(distance_x))
        if distance_y != None and distance_x is None:
            distance_y = (-distance_y)/(10.0+10.0)*(screen_height-0.0)+0.0
            return(int(distance_y))
        if distance_x != None and distance_y != None:
            distance_x = (distance_x)/(10.0+10.0)*(screen_width-0.0)+0.0
            distance_y = (-distance_y)/(10.0+10.0)*(screen_height-0.0)+0.0
            return(int(distance_x), int(distance_y))


    def from_pygame(x=None, y=None):
        """
        Help incomplete, sorry.

        Parameters
        ----------
        NA

        Returns
        ----------
        NA

        Example
        ----------
        NA

        Authors
        ----------
        Dominique Makowski

        Dependencies
        ----------
        - pygame
        """
        if x != None and y == None:
            x =20*x/screen_width - 10
            return(x)
        if x == None and y != None:
            y = -(20*y/screen_height) + 10
            return(y)
        if x != None and y != None:
            x =20*x/screen_width - 10
            y = -(20*y/screen_height) + 10
            return(x, y)

    def to_physical(distance_x=None, distance_y=None, monitor_diagnonal=monitor_diagonal, unit="cm"):
        """
        Help incomplete, sorry.

        Parameters
        ----------
        monitor_diagonal = int
            in inches (24, 27, etc).

        Returns
        ----------
        NA

        Example
        ----------
        NA

        Authors
        ----------
        Dominique Makowski

        Dependencies
        ----------
        None
        """

        if unit=="cm":
            diagonal = monitor_diagonal*2.54

        coef = np.sqrt(((screen_height*screen_height) + (screen_width*screen_width))/(diagonal*diagonal))

        monitor_height = screen_height/coef
        monitor_width = screen_width/coef


        if distance_x != None and distance_y is None:
            distance_x = (distance_x)/(10.0+10.0)*(monitor_width-0.0)+0.0
            return(int(distance_x))
        if distance_y != None and distance_x is None:
            distance_y = (distance_y)/(10.0+10.0)*(monitor_height-0.0)+0.0
            return(int(distance_y))
        if distance_y != None and distance_x != None:
            distance_x = (distance_x)/(10.0+10.0)*(monitor_width-0.0)+0.0
            distance_y = (distance_y)/(10.0+10.0)*(monitor_height-0.0)+0.0
            return(distance_x, distance_y)
    def from_physical(distance_x=None, distance_y=None, monitor_diagonal=monitor_diagonal, unit="cm"):
        """
        Help incomplete, sorry.

        Parameters
        ----------
        monitor_diagonal = int
            in inches (24, 27, etc).

        Returns
        ----------
        NA

        Example
        ----------
        NA

        Authors
        ----------
        Dominique Makowski

        Dependencies
        ----------
        None
        """

        if unit=="cm":
            diagonal = monitor_diagonal*2.54

        coef = np.sqrt(((screen_height*screen_height) + (screen_width*screen_width))/(diagonal*diagonal))

        monitor_height = screen_height/coef
        monitor_width = screen_width/coef


        if distance_x != None and distance_y is None:
            distance_x = (distance_x)*(10.0+10.0)/(monitor_width-0.0)+0.0
            return(int(distance_x))
        if distance_y != None and distance_x is None:
            distance_y = (distance_y)*(10.0+10.0)/(monitor_height-0.0)+0.0
            return(int(distance_y))
        if distance_y != None and distance_x != None:
            distance_x = (distance_x)/(10.0+10.0)*(monitor_width-0.0)+0.0
            distance_y = (distance_y)/(10.0+10.0)*(monitor_height-0.0)+0.0
            return(distance_x, distance_y)
# ==============================================================================
# ==============================================================================
# ==============================================================================
# ==============================================================================
# ==============================================================================
# ==============================================================================
# ==============================================================================
# ==============================================================================
class Font_Cache_Init:
    def __init__(self):
        self.cache = {}  #Initialize an empty cache
    def get(self,font_path,size):
        if not (font_path,int(size)) in self.cache:  #if not in cache,
            if os.path.exists(font_path):  #if the path leads to a font,
                self.cache[font_path,int(size)] = pygame.font.Font(font_path, int(size))  #load this font
            else:
                self.cache[font_path,int(size)] = pygame.font.SysFont(font_path, int(size))  #load a system font
        return(self.cache[font_path,int(size)])
global Font
Font = Font_Cache_Init()  #Create the font object that will update itself with the different loaded fonts

# ==============================================================================
# ==============================================================================
# ==============================================================================
# ==============================================================================
# ==============================================================================
# ==============================================================================
# ==============================================================================
# ==============================================================================
color_list = {
"white":(255,255,255),
"w":(255,255,255),
"black":(0,0,0),
"b":(0,0,0),
"grey":(128,128,128),
"g":(128,128,128),
"raw_red":(255,0,0),
"raw_green":(0,255,0 ),
"raw_blue":(0, 0, 255),

"red":(244,67,54),
"pink":(233,30,99),
"purple":(156,39,176),
"deeppurple":(103,58,183),
"indigo":(63,81,181),
"blue":(33,150,243),
"lightblue":(3,169,244),
"cyan":(0,188,212),
"teal":(0,150,136),
"green":(76,175,80),
"lightgreen":(139,195,74),
"lime":(205,220,57),
"yellow":(255,235,59),
"amber":(255,193,7),
"orange":(255,152,0),
"deeporange":(255,87,34),
"brown":(121,85,72),
"lightgrey":(220,220,220),
"darkgrey":(105,105,105),
"bluegrey":(96,125,139),


"pale_red":(239,154,154),
"pale_pink":(244,143,177),
"pale_purple":(206,147,216),
"pale_deeppurple":(179,157,219),
"pale_indigo":(159,168,218),
"pale_blue":(144,202,249),
"pale_light_blue":(129,212,250),
"pale_cyan":(128,222,234),
"pale_teal":(128,203,196),
"pale_green":(165,214,167),
"pale_lightgreen":(197,225,165),
"pale_lime":(230,238,156),
"pale_yellow":(255,245,157),
"pale_amber":(255,224,130),
"pale_orange":(255,204,128),
"pale_deeporange":(255,171,145),
"pale_brown":(188,170,164),

"blue_shade":[(204,229,255),(153,204,255),(102,178,222),(51,153,255),(0,128,255)],
"red_shade":[(255,204,204),(255,153,153),(255,102,102),(255,51,51),(255,0,0)],
"green_shade":[(204,255,204),(153,255,153),(102,255,102),(51,255,51),(0,255,0)],
"multi_shade":[(255,51,51),(255,51,255),(51,153,255),(51,255,51),(255,153,51)]
}

def color(color):
    """
    Returns an RGB color tuple (or list) from its name.

    Parameters
    ----------
    color = str
        one from the color_list list

    Returns
    ----------
    tuple or list

    Example
    ----------
    >>> import neuropsydia as n
    >>> n.start()
    >>> print(n.color_list)
    >>> print(n.color("blue"))
    >>> n.close()

    Authors
    ----------
    Dominique Makowski

    Dependencies
    ----------
    None
    """
    if isinstance(color,str):
        try:
            return(color_list[color])
        except:
            print("NEUROPSYDIA WARNING: color() was used, however the argument " + str(color) + " was not detected and might cause errors.")
            return(color)
    else:
        return(color)

# ==============================================================================
# ==============================================================================
# ==============================================================================
# ==============================================================================
# ==============================================================================
# ==============================================================================
# ==============================================================================
# ==============================================================================
def cursor(visible=True):
    """
    Set the mouse cursor to visible or invisible.

    Parameters
    ----------
    visible = bool
        True for visible, False for invisible.

    Returns
    ----------
    None

    Example
    ----------
    >>> import neuropsydia as n
    >>> n.start()
    >>> n.cursor(True)
    >>> n.time.wait(2000)
    >>> n.close()

    Authors
    ----------
    The pygame team

    Dependencies
    ----------
    - pygame 1.9.2
    """
    pygame.mouse.set_visible(visible)



# -*- coding: utf-8 -*-
"""
Submodule for the ask() function.
"""
from .path import *
from .core import *
from .write import *

from .core import color as core_color #avoid conflict with arg name



#==============================================================================
#==============================================================================
#==============================================================================
#==============================================================================
#==============================================================================
#======================================================================== ======
#==============================================================================
#==============================================================================
def ask(text="Write something here:", style='light', x=-8, y=0, order=None, size=1.0, color="black", background="white", hide=False, detach_question=False, question_style="light", question_x=0, question_y=0, question_size=1, question_color="black", question_long_text=False, allow=None, allow_length=None, allow_type=None, allow_max=None, allow_NA=True):
    """
    Display a question and get the subject's answer.

    Parameters
    ------------
    text : str
        The question to be displayed.
    style : str
        "body", "light" or "bold".
    order : int
        For series of questions, it's sometimes easier to just specify the order (1, 2 , 3, ...) and the quetsions will appear one under the other.
    x : float
        Horizontal position (from -10 (left) to 10 (right)).
    y : float
        Vertical position of the center (from -10 (bottom) to 10 (top)).
    size : float
        Text size.
    color : str or tuple
        Text color. See `neuropsydia.color()`.
    background : str or tuple
        Background color. See `neuropsydia.color()`.
    hide : bool
        Display "****" (stars) instead of the actual answer.
    detach_question : bool
        If set to true, then the question can be manipulated separately using the parameters below.
    question_style : str
        'body', 'psychometry', 'psychometry_bold', 'light', 'bold', 'title', 'subtitle' or 'end'. Can overwrite other parameters such as position, size or allow. You can also insert the name of a system font, or the path to a specific font.
    question_x : float
        Horizontal position of the question (from -10 (left) to 10 (right)).
    question_y : float
        Vertical position of the question (from -10 (bottom) to 10 (top)).
    question_size : float
        Question size.
    question_color : str
        Question color. See `neuropsydia.color()`.
    question_long_text : bool
        et to True for longer texts on multiple lines. Then, the x and y parameters are not working, but you can jump lines using 'backslash + n' in your text. Some parameters are disabled. Unperfect for now.
    allow : list
        Allow only specific answers (e.g., ["yes", "no"]).
    allow_length : int
        Allow only a specific answer length.
    allow_type : str
        "str", "int" or "float". Allow only a specific type.
    allow_max : int
        When allow_type is int or float, set a maximum.
    allow_NA : bool
        Allow absence of response.

    Returns
    ----------
    answer : str
        Input.

    Example
    ----------
    >>> import neuropsydia as n
    >>> n.start()
    >>> answer = n.ask("Hey, you're good?")
    >>> print(answer)
    >>> n.close()

    Notes
    ----------
    *Authors*

    - Dominique Makowski (https://github.com/DominiqueMakowski)
    - Léo Dutriaux (https://github.com/LeoDutriaux)

    *Dependencies*

    - pygame
    """
    if detach_question is not False:
        write(text, style=question_style, x=question_x, y=question_y, size=question_size, color=question_color, long_text=question_long_text)
        text_new = ''
    else:
        text_new = text + " "

    # Convert size to pygame compatible size
    size = int(size*screen_width/35.0)

    # Get fonts
    if style is 'body':
        font = Font.get(Path.font() + 'RobotoRegular.ttf', size)
    elif style is 'light':
        font = Font.get(Path.font() + 'RobotoLight.ttf', size)
    elif style is 'bold':
        font = Font.get(Path.font() + 'RobotoBold.ttf', size)
    else:
        font = Font.get(style, size)

    # Adjust y position depending on order
    if order is not None:
        y = 5 - (order*1.25)


    # Convert coordinates to pygame compatible coordinates
    x, y = Coordinates.to_pygame(x=x, y=y)


    user_input = ''
    loop = True
    while loop is True:
        surface = font.render(text_new, 1, core_color(color))
        size = surface.get_size()
        pygame.draw.rect(screen, core_color(background), (x,y,size[0],size[1]))
        screen.blit(surface, (x,y))
        pygame.display.flip()

        # wait for user input
        answer = response(get_RT=False)

        if answer is "SPACE":
            answer = "_"
        if answer is not "ENTER":
            if answer is "ESCAPE":
                if allow_NA is True:
                    return("NA")
                else:
                    user_input = ''
                    warning_text = text_new+'    incorrect (input required)'
                    warning = 1
            if answer is pygame.K_BACKSPACE:
                if user_input != '':
                    user_input = user_input[:len(user_input)-1]
                    text_new = text_new[:len(text_new)-1]
                    pygame.draw.rect(screen, core_color(background), (x,y,size[0],size[1]))
            else:
                user_input = user_input + str(answer)
                if hide is True:
                    text_new = text_new + "*"
                else:
                    text_new = text_new + str(answer)
        else:
            if user_input is '':
                user_input = "NA"
                if allow_NA is True:
                    return(user_input)
                else:
                    user_input = ''
                    warning_text = text_new+'    incorrect (input required)'
                    warning = 1
            elif allow is not None or allow_length is not None or allow_type is not None:
                warning = 0
                if allow is not None and user_input not in list(allow):
                    warning_text = text_new+'    incorrect (answer not allowed)'
                    warning = 1
                if isinstance(allow_length, int) and len(user_input) != allow_length:
                    warning_text = text_new+'    incorrect (' +str(allow_length)+ ' characters required)'
                    warning = 1
                if allow_max is not None and float(user_input) > float(allow_max):
                    warning_text = text_new+'    incorrect (max = ' + str(allow_max) + ')'
                    warning = 1
                    allow_type = "int"
                if allow_type is not None:
                    if allow_type is "int":
                        try:
                            user_input = int(user_input)
                        except ValueError:
                            warning_text = text_new + '    incorrect (wrong type)'
                            warning = 1
                    if allow_type is "float":
                        try:
                            user_input = float(user_input)
                        except ValueError:
                            warning_text = text_new + '    incorrect (wrong type)'
                            warning = 1

                if warning == 1:
                    surface = font.render(warning_text, 1, core_color(color))
                    size = surface.get_size()
                    pygame.draw.rect(screen,  core_color(background), (x,y,size[0],size[1]))
                    screen.blit(surface, (x,y))
                    refresh()
                    time.wait(700)
                    pygame.draw.rect(screen,  core_color(background), (x,y,size[0],size[1]))
                    refresh()
                else:
                    return(user_input)
            else:
                return(user_input)








# -*- coding: utf-8 -*-
"""
Module initializing screen object and screen values.
"""
import pygame
from .path import *

# Change neuropsydia.screen to "__main__" When building API documentation. This is to avoid sphinx to run this code, otherwise the documentations fails to be built. "neuropsydia.screen" to make it work.

if __name__ == "neuropsydia.screen":
    # Add icon to the window
    pygame.display.set_icon(pygame.image.load(Path.logo() + 'icon.png'))

    # Create screen
    screen = pygame.display.set_mode((0,0), pygame.SRCALPHA | pygame.FULLSCREEN | pygame.DOUBLEBUF | pygame.HWSURFACE)

    # Name the screen
    pygame.display.set_caption('Neuropsydia')

    # Get screen dimensions
    screen_width, screen_height = screen.get_size()

    # Initialize monitor diagonal size
    monitor_diagonal = 24  # inch

else:
    screen = "Placeholder"
    screen_width, screen_height = 0, 0
    monitor_diagonal = 24


# -*- coding: utf-8 -*-
class Trigger():
    """
    A class object to send trigger via TTL, ethernet or stimtracker.

    Parameters
    ----------
    TTL = bool, optional
        Send trigger through the parallel port.
    photosensor = str, optional
        "white" or "black" for the color of the rectangle.
    photosensor_position = str, optional
        "bottomleft", "bottomright", "topleft" or "topright" for its position.
    stimtracker = bool, optional
        Send trigger through a stimtracker.
    stimtracker_duration = float, optional
        Time for the stimtracker trigger to last (in seconds).

    Returns
    ----------
    None

    Example
    ----------
    >>> import neuropsydia as n
    >>> n.start()
    >>> trigger = n.Trigger()
    >>> trigger.start()
    >>> trigger.stop()
    >>> n.close()

    Authors
    ----------
    Dominique Makowski

    Dependencies
    ----------
    - ctypes
    - pyxid
    """
    def __init__(self, TTL=True, photosensor=None, photosensor_position="bottomleft"):
        self.photosensor = photosensor
        self.photosensor_position = photosensor_position
        if self.method=="TTL":
            try:
                from ctypes import windll
                global io
                io = windll.dlportio # requires dlportio.dll !
            except:
                print("NEUROPSYDIA WARNING: Trigger(): The parallel port couldn't be opened")

    def start(self, trigger=1, port=0x378, lines=1):
        """
        Send the trigger.

        Parameters
        ----------
        trigger = int, optional
            What trigger to send (TTL).
        port = binary, optional
            Port address (TTL).
        lines = int, optional
            Lines to activate (stimtracker).

        Returns
        ----------
        None

        Example
        ----------
        >>> import neuropsydia as n
        >>> n.start()
        >>> trigger = n.Trigger()
        >>> trigger.start()
        >>> trigger.stop()
        >>> n.close()

        Authors
        ----------
        Dominique Makowski

        Dependencies
        ----------
        - ctypes
        - pyxid
        """
        if self.photosensor != None:
            if self.photosensor_position == "bottomleft":
                rectangle(x=-10, y=-10, width=screen_width/screen_width, height=screen_width/screen_height, thickness=0, fill_color=self.photosensor)
            refresh()
        if self.method == "TTL":
            try:
                io.DlPortWritePortUchar(port, trigger)
            except:
                print('NEUROPSYDIA WARNING: Trigger.start(): Failed to send trigger!')

    def stop(self, trigger=0, port=0x378):
        """
        Return to baseline (for TTL only).

        Parameters
        ----------
        trigger = int, optional
            What trigger to send (TTL).
        port = binary, optional
            Port address (TTL).

        Returns
        ----------
        None

        Example
        ----------
        >>> import neuropsydia as n
        >>> n.start()
        >>> trigger = n.Trigger()
        >>> trigger.start()
        >>> trigger.stop()
        >>> n.close()

        Authors
        ----------
        Dominique Makowski

        Dependencies
        ----------
        - ctypes
        - pyxid
        """
        if self.method == "TTL":
            try:
                io.DlPortWritePortUchar(port, trigger)
            except:
                print('NEUROPSYDIA WARNING: Trigger.stop(): Failed to send trigger!')


# -*- coding: utf-8 -*-
import inspect

class Path:
    def root():
        return(inspect.getfile(Path).split("path.py")[0])
    def logo():
        return(inspect.getfile(Path).split("path.py")[0] + r"files/logo/")
    def binary():
        return(inspect.getfile(Path).split("path.py")[0] + r"files/binary/")
    def font():
        return(inspect.getfile(Path).split("path.py")[0] + r"files/font/")


# -*- coding: utf-8 -*-
from .path import *
from .core import *
from .image import *
from .write import *
from .miscellaneous import *

# ==============================================================================
# ==============================================================================
# ==============================================================================
# ==============================================================================
# ==============================================================================
# ==============================================================================
# ==============================================================================
# ==============================================================================
def start(open_window=True):
    """
    Initialize all the components of Neuropsydia. Always at the beginning of a neuropsydia script.

    Parameters
    ----------
    open_window = bool
        should it open the pygame's window or close it immediatly (useful when using neuropsydia for something else than experiments, e.g., statistics)

    Returns
    ----------
    None

    Example
    ----------
    >>> import neuropsydia as n
    >>> n.start()
    >>> n.close()

    Authors
    ----------
    Dominique Makowski

    Dependencies
    ----------
    - pygame 1.9.2
    """
    pygame.init()
    pygame.mouse.set_visible(False)
    pygame.event.set_blocked(pygame.KEYDOWN)


    if open_window == False:
        pygame.event.set_allowed(pygame.KEYDOWN)
        pygame.mouse.set_visible(True)
        pygame.quit()

    else:
        newpage("black", auto_refresh=False)
        write('Initialisation...', size=0.8, y=-9, color="lightblue")
        refresh()

        preloaded = {}
        preloaded = preload("Neuropsydia_PSY_blue", extension = ".png", y= -2.5, size=14, cache = preloaded, path = Path.logo())
        preloaded = preload("Neuropsydia_TEXT_white", extension = ".png", y= 7, size=5, cache = preloaded, path = Path.logo())
        preloaded = preload("Neuropsydia_HEAD_white", extension = ".png", y= -2.5, size=14, cache = preloaded, path = Path.logo())

        newpage("black", auto_refresh=False)
        image("Neuropsydia_TEXT_white", extension = ".png", y= 7, size=5, cache = preloaded, path = Path.logo())
        image("Neuropsydia_PSY_blue", extension = ".png", y= -2.5, size=14, cache = preloaded, path = Path.logo())
        write('Press ENTRER to continue.', size=0.8, y=-9, color="white", allow="ENTER")

        # Fade
        for i in range(0,100,2):
            newpage("black", auto_refresh=False)
            image("Neuropsydia_HEAD_white", extension = ".png", y= -2.5, size=14, cache = preloaded, path = Path.logo())
            newpage("black", opacity = 100 - i, auto_refresh=False)
            image("Neuropsydia_TEXT_white", extension = ".png", y= 7, size=5, cache = preloaded, path = Path.logo())
            image("Neuropsydia_PSY_blue", extension = ".png", y= -2.5, size=14, cache = preloaded, path = Path.logo())
            refresh()
            
        time.wait(800)
        newpage("white", fade=True)
        refresh()

# ==============================================================================
# ==============================================================================
# ==============================================================================
# ==============================================================================
# ==============================================================================
# ==============================================================================
# ==============================================================================
# ==============================================================================
def close():
    """
    A clean closing of all the components of Neuropsydia. Always at the end of a neuropsydia script.

    Parameters
    ----------
    None

    Returns
    ----------
    None

    Example
    ----------
    >>> import neuropsydia as n
    >>> n.start()
    >>> n.close()

    Authors
    ----------
    Dominique Makowski

    Dependencies
    ----------
    - pygame 1.9.2
    """
    newpage("black", auto_refresh=False)
    write("Please wait...",color="white")
    refresh()

    preloaded = {}
    preloaded = preload("Neuropsydia_TEXT_white", extension = ".png", y= 5.5, size=4, cache = preloaded, path = Path.logo())
    preloaded = preload("Neuropsydia_HEAD_white", extension = ".png", y= -3, size=12.5, cache = preloaded, path = Path.logo())
    preloaded = preload("Neuropsydia_PSY_blue", extension = ".png", y= -3, size=12.5, cache = preloaded, path = Path.logo())
    preloaded = preload('N', extension = ".png", x=7, y=-8, size=2.5, cache = preloaded, path = Path.logo())
    preloaded = preload("Python", extension = ".png", x=-7, y=-8, size=2.5, cache = preloaded, path = Path.logo())

    for i in range(0,100,2):
        newpage("black", auto_refresh=False)
        write("Thank you for using", style="light", y=8.75, size=1, color="white")
        image("Neuropsydia_TEXT_white", extension = ".png", y= 5.5, size=4, cache = preloaded, path = Path.logo())
        image("Neuropsydia_HEAD_white", extension = ".png", y= -3, size=12.5, cache = preloaded, path = Path.logo())
        image("Neuropsydia_PSY_blue", extension = ".png", y= -3, size=12.5, cache = preloaded, path = Path.logo())
        image('N', extension = ".png", x=7, y=-8, size=2.5, cache = preloaded, path = Path.logo())
        image("Python", extension = ".png", x=-7, y=-8, size=2.5, cache = preloaded, path = Path.logo())
        newpage("black", opacity = 100-i, auto_refresh=False)
        refresh()
    time.wait(1500)

    pygame.event.set_allowed(pygame.KEYDOWN)
    pygame.mouse.set_visible(True)
    pygame.quit()





# -*- coding: utf-8 -*-
import PIL
import numpy as np

from .path import *
from .core import *


# ==============================================================================
# ==============================================================================
# ==============================================================================
# ==============================================================================
# ==============================================================================
# ==============================================================================
# ==============================================================================
# ==============================================================================
class Preload():
    """
    Preload images.

    Parameters
    ----------
    NA

    Returns
    ----------
    NA

    Example
    ----------
    NA

    Authors
    ----------
    Dominique Makowski

    Dependencies
    ----------
    - pygame
    - PIL
    """
    def __init__(self, files, x=0, y=0, size=1.0, path='', extension=''):
        self.dict_PIL = {}

        if isinstance(files, list) is False:
            list_files = []
            list_files.append(files)
        else:
            list_files = files
        n_files = len(list_files)

        if isinstance(x, list) is False:
            list_x = []
            list_x.append(x)
        else:
            list_x = x
        n_x = len(list_x)
        if n_x != n_files:
            list_files = list_files*n_x
            list_x = list_x*n_files
            list_files = list_files[:max([n_files,n_x])]
            list_x = list_x[:max([n_files,n_x])]

        if isinstance(y, list) is False:
            list_y = []
            list_y.append(y)
        else:
            list_y = y
        n_y = len(list_y)
        if n_y != n_files:
            list_files = list_files*n_y
            list_y = list_y*n_files
            list_files = list_files[:max([n_files,n_y])]
            list_y = list_y[:max([n_files,n_y])]

        if isinstance(size, list) is False:
            list_size = []
            list_size.append(size)
        else:
            list_size = size
        n_size = len(list_size)
        if n_size != n_files:
            list_files = list_files*n_size
            list_size = list_size*n_files
            list_files = list_files[:max([n_files,n_size])]
            list_size = list_size[:max([n_files,n_size])]



        for n, name in enumerate(list_files):
            if os.path.isfile(path+name+extension) is False:
                print("NEUROPSYDIA WARNING: preload_image(): wrong name, path or extension")
                if os.path.isfile(path+name+'.png') is True:
                    print("NEUROPSYDIA WARNING: preload_image(): image found adding '.png'")
                    extension = ".png"
                if os.path.isfile(path+name+'.jpg') is True:
                    print("NEUROPSYDIA WARNING: preload_image(): image found adding '.jpg'")
                    extension = ".jpg"

            image = PIL.Image.open(path+name+extension, "r")
            self.dict_PIL["%s%s%s_%s%s%s" %(path, name, extension, str(list_x[n]), str(list_y[n]), str(list_size[n]))] = image


    def to_PIL(self):
        return(self.dict_PIL)

    def to_arrays(self):
        dict_of_arrays = {}
        for i in self.cache:
            dict_of_arrays[i] = np.array(self.cache[i])
        return(dict_of_arrays)
# ==============================================================================
# ==============================================================================
# ==============================================================================
# ==============================================================================
# ==============================================================================
# ==============================================================================
# ==============================================================================
# ==============================================================================
def preload(file, x=0, y=0, cache=None, path='', extension='', size=1.0, unit="n", scale_by="height", fullscreen=False, rotate=0, scramble=False, compress=False, compression=0, opacity=100, key=None, monitor_diagonal=monitor_diagonal):
    """
    Preload images.

    Parameters
    ----------
    NA

    Returns
    ----------
    NA

    Example
    ----------
    NA

    Authors
    ----------
    Dominique Makowski

    Dependencies
    ----------
    - pygame
    - PIL
    """
    if isinstance(file, list) is False:
        files = []
        files.append(file)
    else:
        files = file

    if cache != None:
        dictionary = cache
    else:
        dictionary = {}


    for i in files:
        #Check existence of the given files
        if os.path.isfile(path+i+extension) is False:
            print("NEUROPSYDIA WARNING: preload_image(): wrong name, path or extension")
            if os.path.isfile(path+i+'.png') is True:
                print("NEUROPSYDIA WARNING: preload_image(): image found adding '.png'")
                extension = ".png"
            if os.path.isfile(path+i+'.jpg') is True:
                print("NEUROPSYDIA WARNING: preload_image(): image found adding '.jpg'")
                extension = ".jpg"


        image = PIL.Image.open(path+i+extension, "r")
        w, h = image.size

        scale_by_original = scale_by
        if fullscreen is False:
            if scale_by == "greater":
                if h > w:
                    scale_by = "height"
                else:
                    scale_by = "width"
            if unit == "n":  # size=1 means height is equals to 1
                if scale_by=="height":
                    new_w = int(w/h*size*screen_height/20.0)
                    new_h = int(h/h*size*screen_height/20.0)
                if scale_by=="width":
                    new_w = int(w/w*size*screen_height/20.0)
                    new_h = int(h/w*size*screen_height/20.0)
            if unit == "cm" or unit == "inch":
                if scale_by=="height":
                    new_h = int(Coordinates.to_pygame(distance_y=-1*Coordinates.from_physical(distance_y=size, unit=unit, monitor_diagonal=monitor_diagonal)))
                    new_w = int(new_h * w / h)
                if scale_by=="width":
                    new_w = int(Coordinates.to_pygame(distance_x=Coordinates.from_physical(distance_x=size, unit=unit, monitor_diagonal=monitor_diagonal)))
                    new_h = int(new_w * h / w)
            if scale_by_original == "greater":
                scale_by = "greater"

        else:
            if w > h:
                new_w = int(w/w*screen_width)
                new_h = int(h/w*screen_width)
            else:
                new_w = int(w/h*screen_height)
                new_h = int(h/h*screen_height)
        image = image.resize((new_w, new_h),PIL.Image.ANTIALIAS)
        image = image.rotate(rotate)

        if scramble == True:
            array = np.array(image)
            array = np.random.permutation(array)
            image = PIL.Image.fromarray(array)
        if compress == True:
            image = image.resize((int(w*(100-compression)/100), int(h*(100-compression)/100)), PIL.Image.ANTIALIAS)
            image.save(path + "temp.jpg", quality=10, optimize=True)
            image = PIL.Image.open(path + "temp.jpg", "r")
            image = image.resize((new_w, new_h),PIL.Image.ANTIALIAS)
            os.remove(path + "temp.jpg")

        if opacity != 100:
            image = image.convert('RGBA')
            image.putalpha(int(opacity * 255 / 100))


        image = pygame.image.fromstring(image.tobytes(), image.size, image.mode)


        if key == None:
            dictionary[path + file + '_' + str(size) + '_' + str(unit) + '_' + str(scale_by) + '_' + str(rotate) + '_' + str(opacity) + '_' + str(monitor_diagonal) + '_' + extension] = image
        else:
            dictionary[key] = image




    #return
    return(dictionary)



#==============================================================================
#==============================================================================
#==============================================================================
#==============================================================================
# # # # --- image ---
#version : 1.0
#display an image.
#==============================================================================
#==============================================================================
#==============================================================================
#==============================================================================
def image(file, x=0, y=0, cache=None, path='', extension='', size=1.0, unit="n", scale_by="height", fullscreen=False, rotate=0, scramble=False, background=None, compress=False, compression=0, allow=None, wait=None, opacity=100, monitor_diagonal=monitor_diagonal):
    """
    Display an image on screen.

    Parameters
    -----------
    file : str
        Image filename.
    x : float
        Horizontal position of image center (from -10 (left) to 10 (right)).
    y : float
        Vertical position of image center (from -10 (bottom) to 10 (top)).
    cache : dict
        Cache of preloaded files.
    path : str
        File's path.
    extension : str
        File's extension.
    size : float
        Image size.
    unit : str
        Size unit. 'n' for neuropsydia's space, can be 'cm'.
    scale_by : str
        'height' or 'width'.
    fullscreen : bool
        Fullscreen. Disable the size parameter.
    rotate : int
        Rotation angle.
    scramble : bool
        Scramble (randomize pixels).
    background : str
        Background colour.
    compress : bool
        Enable compression.
    compression : int
        Compression rate.
    allow : list
        Wait until a specific key is pressed (e.g., "ENTER", ["LEFT", "RIGHT"] or "any").
    wait : int
        Wait time (in milliseconds).
    opacity : 100
        Opacity (in percentage).
    monitor_diagonal : int
        Monitor size (in inches).

    Returns
    ----------
    cache : dict
        The updated cache.

    Example
    ----------
    >>> import neuropsydia as n
    >>>
    >>> n.start()
    >>> for file in ["img1.png", "img2.png"]:
    >>>    n.newpage()
    >>>    n.image(file)
    >>>    n.refresh()
    >>>    n.time.wait(1000)
    >>> n.close()

    Notes
    ----------
    *Authors*

    - Dominique Makowski (https://github.com/DominiqueMakowski)
    - Léo Dutriaux (https://github.com/LeoDutriaux)

    *Dependencies*

    - pygame
    - PIL
    """
    if background is not None:
        newpage(background, auto_refresh=False)

    if cache is None:
        cache = preload(file=file, cache=cache, path=path, extension=extension, size=size, unit=unit, scale_by=scale_by, fullscreen=fullscreen, rotate=rotate, scramble=scramble, compress=compress, compression=compression, opacity=opacity, monitor_diagonal=monitor_diagonal)

        image = cache[path + file + '_' + str(size) + '_' + str(unit) + '_' + str(scale_by) + '_' + str(rotate) + '_' + str(opacity) + '_' + str(monitor_diagonal) + '_' + extension]

    else:
        try:
            image = cache.Cache[path + file + '_' + str(size) + '_' + str(unit) + '_' + str(scale_by) + '_' + str(rotate) + '_' + str(opacity) + '_' + str(monitor_diagonal) + '_' + extension]
        except:
            try:
                image = cache[path + file + '_' + str(size) + '_' + str(unit) + '_' + str(scale_by) + '_' + str(rotate) + '_' + str(opacity) + '_' + str(monitor_diagonal) + '_' + extension]
            except:
                print('NEUROPSYDIA ERROR: image(): file not in given cache: ' + file)
                cache = preload(file=file, cache=cache, path=path, extension=extension, size=size, unit=unit, scale_by=scale_by, fullscreen=fullscreen, rotate=rotate, scramble=scramble, compress=compress, compression=compression, opacity=opacity, monitor_diagonal=monitor_diagonal)
                image = cache[path + file + '_' + str(size) + '_' + str(unit) + '_' + str(scale_by) + '_' + str(rotate) + '_' + str(opacity) + '_' + str(monitor_diagonal) + '_' + extension]


    x, y = Coordinates.to_pygame(x=x, y=y)
    rectangle = image.get_rect()
    rectangle.center = (x, y)
    screen.blit(image, rectangle)

    #In case one must wait until something
    if allow is not None or wait is not None:
        refresh()
        if allow is None and wait is not None:
            time.wait(wait)
            return(wait)
        if allow is not None:
            return(response(allow=allow, time_max=wait))
    else:
        return(cache)








# -*- coding: utf-8 -*-

from setuptools import find_packages, setup

setup(
    name='myNeurospydia',
    packages=find_packages(),
    version='0.1.1',
    description='Partially fixed version of Neuropsydia library, a Python module for creating experiments, tasks and questionnaires',
    author='Laborde',
    license='ENS_Paris_Saclay',
    install_requires=['pygame',
                      'numpy',
                      'neurokit',
                      'pandas',
                      'datetime',
                      'webbrowser',
                      'scipy'],
)


__version__ = "1.0.3"

from .path import *
from .core import *
from .miscellaneous import *
from .write import *
from .image import *
from .start import *
from .scale import *
from .ask import *
from .choice import *
from .meta import *
from .io import *
from .procedures import *

